/*****************************************************
Project : PWM Bipolar Locomotive Controller;    
Version : 1.00
Date    : 08.01.2007
Author  : Yuri V. Kalashnikov; . (_)                            
Company : KRTPC 
Comments: Bipolar PWM Power Supply Controller
 for railway models;       

      ""   "   ? . "


Chip type           : ATtiny13
Clock frequency     : 9,600000 MHz internal RC option;   RC    9.6
                      CKSEL1..0 = 10;          10
Memory model        : Tiny
External SRAM size  : 0
Data Stack size     : 16
*****************************************************/

#include <tiny13.h>
#include <delay.h>

void main(void)
{
int k,i;
int AH, AL;

// Crystal Oscillator division factor: 1
#pragma optsize-
CLKPR=0x80;
CLKPR=0x00;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

// Input/Output Ports initialization
// Port B initialization
// Func5=In Func4=Out Func3=In Func2=In Func1=Out Func0=Out 
// State5=T State4=0 State3=T State2=T State1=0 State0=0 
PORTB=0x00;
DDRB=0x13;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: Timer 0 Stopped
// Mode: Normal top=FFh
// OC0A output: Disconnected
// OC0B output: Disconnected
TCCR0A=0x00;
TCCR0B=0x00;
TCNT0=0x00;
OCR0A=0x00;
OCR0B=0x00;

// External Interrupt(s) initialization
// INT0: Off
// Interrupt on any change on pins PCINT0-5: Off
GIMSK=0x00;
MCUCR=0x00;

// Timer/Counter 0 Interrupt(s) initialization
TIMSK0=0x00;

// Analog Comparator initialization
// Analog Comparator: Off
ACSR=0x80;
ADCSRB=0x00;

//    ,      .5
ADMUX=0x03; // Vcc as VRef, ADC connect to PB3 pin

//   ,    = 110   9.6   = 150
      50...200
ADCSRA=86;  // ADC enable, ADC CLK divider=0b110 (9600/64=150kHz)

k=500; //start position = "STOP" ;  -   ""

while (1)
  { //   
    
       //  
       ADCSRA=0xC6; //START conversion
       
	//       


       if ((k>412) && (k<612)) // CENTRAL "STOP" trimmer position
        //     ""    .1  .4   0,   
         {
           PORTB.0=0; // LED "STOP" is lighting;  
           PORTB.1=0;
           PORTB.4=0;
         }
         else  //   412<k<612
           {
             PORTB.0=1; // LED "STOP" is OFF;  
             
             if (k<=412)  // RIGHT-turned trimmer position;   0<k<=412   
               {
                 k=412-k;

                 // Pulse Width Modulation cycles "FORWARD";      .4     
                 //  412 ,  0    1,    412  0
                 for(i=1; i<=k; i=i+1)
                   {
                     PORTB.4=1;
                     delay_us(10); //      
                   }
                 for(i=k; i<=412; i=i+1)
                   {
                     PORTB.4=0;
                     delay_us(10);
                   }    
               
               } //(k<=412)         
               
             if(k>=612)    // LEFT-turned trimmer position;     612
               {
                 k=k-612;
                 // Pulse Width Modulation cycles "BACK";    .1 -      
                 for(i=1; i<=k; i=i+1)
                   {
                    PORTB.1=1;
                    delay_us(10); 
                   }
                 for(i=k; i<=412; i=i+1)
                   {
                     PORTB.1=0;
                     delay_us(10);
                   }    
               } //(k>=612)
          
           } // else
       
       //        
       //     
       AL=ADCL; // extract low part ADC result;   
       AH=ADCH; // extract high part ADC result;   
       k=AH*256; // shift high part to most significant byte;       
       k=k+AL; // add low byte;    
  };// while
}   // main
// Ѩ